#include <avr/io.h>
#include "device.h"
#include "display.h"
#include "button.h"
#include "rtc.h"
#include "ir.h"
#include "bcd.h"
#include "app_clk.h"

/* Digital clock */

typedef enum {
    CLK_INIT,
    CLK_SETUP,
    CLK_LOCK
} CLK_STATE;

static CLK_STATE state = CLK_INIT;

static uint8_t unlock_counter = 0;
static uint8_t unlock_change = 0;
static uint8_t lock_counter = 0;

static void CLK_Setup(void) {
    lock_counter = 0;
    state = CLK_SETUP;
    DISPLAY_SetBrightness(10);
    DISPLAY_ButtonBacklight(SW1|SW2|SW3|SW4);
}

static void CLK_Refresh() {
    RTC_TIME time = RTC_GetTime();
    uint8_t hour = bin2bcd(time.hour);
    uint8_t minute = bin2bcd(time.minute);
    DISPLAY_Clear();
    if(hour < 0x10) {
        DISPLAY_Print(X_(hour), 3);
        DISPLAY_Print(_COLON_, 7);
        DISPLAY_Print(_X(minute), 11);
        DISPLAY_Print(X_(minute), 17);
    } else {
        DISPLAY_Print(_X(hour), 0);
        DISPLAY_Print(X_(hour), 6);
        DISPLAY_Print(_COLON_, 10);
        DISPLAY_Print(_X(minute), 14);
        DISPLAY_Print(X_(minute), 20);
    }
}

static void CLK_ButtonPressed(uint8_t button_state) {
    (void)button_state; //unused
    switch(state) {
        case CLK_INIT: CLK_Setup(); break;
        default: break;
    }
}

static void CLK_ButtonReleased(uint8_t button_changed) {
    switch(state) {
        case CLK_SETUP:
            lock_counter = 0;
            switch(button_changed) {
                case SW1: RTC_ChangeHour(+1); CLK_Refresh(); break;
                case SW2: RTC_ChangeHour(-1); CLK_Refresh(); break;
                case SW3: RTC_ChangeMinute(+1); CLK_Refresh(); break;
                case SW4: RTC_ChangeMinute(-1); CLK_Refresh(); break;
            }
            break;
        case CLK_LOCK:
            unlock_change = 1;
            if(++unlock_counter == 3) {
                unlock_counter = 0;
                unlock_change = 0;
                CLK_Setup();
            }
            break;
        default: break;
    }
}

static uint8_t blink = 0;

static void CLK_Tick(void) {
    switch(state) {
        case CLK_SETUP:
            if(++lock_counter == 5) {
                state = CLK_LOCK;
            }
            break;
        case CLK_INIT:
            CLK_Refresh();
            if(++blink & 0x01) { DISPLAY_SetBrightness(10); }
            else { DISPLAY_SetBrightness(2); }
            break;
        case CLK_LOCK:
            CLK_Refresh();
            DISPLAY_CheckBrightness();
            DISPLAY_ButtonBacklight(0);
            if(unlock_change == 1) {
                unlock_change = 0;
            } else if(unlock_counter > 0) {
                unlock_counter--;
            }
            break;
    }
}

void CLK_Init(void) {
    BUTTON_Activate();
    IR_Activate();
    BUTTON_Pressed = CLK_ButtonPressed;
    BUTTON_Released = CLK_ButtonReleased;
    RTC_Tick = CLK_Tick;
    if(state == CLK_SETUP) { state = CLK_LOCK; }
    CLK_Refresh();
}
