#include <avr/io.h>
#include "display.h"
#include "device.h"
#include "button.h"
#include "bcd.h"
#include "rtc.h"
#include "ir.h"
#include "app.h"
#include "app_irt.h"

/* Infrared remote control tester */

static IR_PROTOCOL protocol = IR_UNKNOWN;
static uint8_t init_counter = 0;
static uint8_t skip = 0;
static uint8_t packed_received = 0;

static void IRT_Display(void) {
    DISPLAY_Clear();
    switch(BUTTON_RAW_STATE) {
        case SW1:
            DISPLAY_Print(_C_,  4);
            DISPLAY_Print(_L_, 10);
            DISPLAY_Print(_K_, 16);
            return;
        case SW2:
            DISPLAY_Print(_T_,  4);
            DISPLAY_Print(_M_, 10);
            DISPLAY_Print(_P_, 16);
            return;
        case SW3:
            DISPLAY_Print(_U_,  4);
            DISPLAY_Print(_R_, 10);
            DISPLAY_Print(_T_, 16);
            return;
    }
    switch(protocol) {
    case IR_UNKNOWN:
        DISPLAY_Print(_I_,  7);
        DISPLAY_Print(_R_, 12);
        break;
    case IR_RC5:
        DISPLAY_Print(_R_,  4);
        DISPLAY_Print(_C_, 10);
        DISPLAY_Print( 5 , 16);
        break;
    case IR_NEC:
        DISPLAY_Print(_N_,  4);
        DISPLAY_Print(_E_, 10);
        DISPLAY_Print(_C_, 16);
        break;
    case IR_KAS:
        DISPLAY_Print(_K_,  4);
        DISPLAY_Print(_A_, 10);
        DISPLAY_Print(_S_, 16);
        break;
    }
}

static void IRT_ButtonPressed(uint8_t button_state) {
    (void)button_state; //unused
    packed_received = 0;
    IRT_Display();
}

static void IRT_ButtonReleased(uint8_t button_changed) {
    IRT_Display();
    if(packed_received) { return; }
    switch(button_changed) {
        case SW1: APP_SaveIrCode(APP_CLK, 0); break;
        case SW2: APP_SaveIrCode(APP_TMP, 0); break;
        case SW3: APP_SaveIrCode(APP_URT, 0); break;
    }
}

static void IRT_Tick(void) {
    if(!skip) {
        IRT_Display();
        if(++init_counter >= 3) {
            init_counter = 0;
            protocol = IR_UNKNOWN;
        }
    }
    skip = 0;
    DISPLAY_CheckBrightness();
}

static void IRT_SignalAppear(void) {
    DISPLAY_Dot_ON();
    skip = 1;
}

static void IRT_SignalDisappear(void) {
    init_counter = 0;
    DISPLAY_Dot_OFF();
}

static void IRT_PacketReceived(IR_PACKET packet) {
    protocol = packet.protocol;
    DISPLAY_Clear();
    DISPLAY_Print(_X(packet.address), 0);
    DISPLAY_Print(X_(packet.address), 6);
    DISPLAY_Print(_X(packet.command), 14);
    DISPLAY_Print(X_(packet.command), 20);
    packed_received++;
    if(packed_received > 1) { return; }
    uint16_t code = (packet.address<<8) | packet.command;
    switch(BUTTON_RAW_STATE) {
        case SW1: APP_SaveIrCode(APP_CLK, code); break;
        case SW2: APP_SaveIrCode(APP_TMP, code); break;
        case SW3: APP_SaveIrCode(APP_URT, code); break;
    }
}

void IRT_Init(void) {
    BUTTON_Activate();
    BUTTON_Pressed = IRT_ButtonPressed;
    BUTTON_Released = IRT_ButtonReleased;
    IR_Activate();
    IR_SignalAppear = IRT_SignalAppear;
    IR_SignalDisappear = IRT_SignalDisappear;
    IR_PacketReceived = IRT_PacketReceived;
    RTC_Tick = IRT_Tick;
    IRT_Display();
}
