#include <avr/io.h>
#include "display.h"
#include "device.h"
#include "bcd.h"
#include "rtc.h"
#include "ir.h"
#include "onewire.h"
#include "ds18B20.h"
#include "app_tmp.h"

/* Digital thermometer */

static uint8_t skip = 0;

static void TMP_Unknown(void) {
    DISPLAY_Clear();
    DISPLAY_Print(_QUESTION_MARK_, 4);
    DISPLAY_Print(_DEGREE_, 10);
    DISPLAY_Print(_C_, 16);
}

static void TMP_DeviceNotPresent(void) {
    DISPLAY_Clear();
    DISPLAY_Print(_N_, 7);
    DISPLAY_Print(_C_, 13);
}

static void TMP_Tick(void) {
    if(skip) { skip = 0; return; }
    DS18B20_ReadTemperature();
    DISPLAY_CheckBrightness();
}

static void TMP_ReadComplete(void) {
    int16_t temp = DS18B20_GetTemperature();
    temp >>= 7;
    uint8_t sign = 0;
    if(temp < 0) {
        sign = 1;
        temp = -temp;
    }
    uint8_t half = temp & 0x01;
    uint8_t value = temp >> 1;
    DISPLAY_Clear();
    if(value > 125) {
        TMP_Unknown();
        return;
    }
    if(value > 99) {
        value -= 100;
        value = bin2bcd(value);
        DISPLAY_Print(1, 1);
        DISPLAY_Print(_X(value), 7);
        DISPLAY_Print(X_(value), 13);
        DISPLAY_Print(_DEGREE_, 19);
        return;
    }
    value = bin2bcd(value);
    if(value < 0x10) {
        if(sign) {
            DISPLAY_Print(_MINUS_, 1);
            DISPLAY_Print(X_(value), 5);
            DISPLAY_Print(_DOT_, 9);
            DISPLAY_Print(half*5, 13);
            DISPLAY_Print(_DEGREE_, 19);
        } else {
            DISPLAY_Print(X_(value), 0);
            DISPLAY_Print(_DOT_, 4);
            DISPLAY_Print(half*5, 8);
            DISPLAY_Print(_DEGREE_, 14);
            DISPLAY_Print(_C_, 20);
        }
    } else {
        if(sign) {
            DISPLAY_Print(_MINUS_, 0);
            DISPLAY_Print(_X(value), 3);
            DISPLAY_Print(X_(value), 9);
            DISPLAY_Print(_DEGREE_, 14);
            DISPLAY_Print(_C_, 20);
        } else {
            DISPLAY_Print(_X(value), 1);
            DISPLAY_Print(X_(value), 7);
            DISPLAY_Print(_DEGREE_, 13);
            DISPLAY_Print(_C_, 19);
        }
    }
}

void TMP_Init(void) {
    IR_Activate();
    RTC_Tick = TMP_Tick;
    DS18B20_Activate();
    DS18B20_ReadComplete = TMP_ReadComplete;
    DS18B20_ReadError = TMP_Unknown;
    ONEWIRE_DeviceNotPresent = TMP_DeviceNotPresent;
    DS18B20_ReadTemperature();
    skip = 1;
}
